@@ -1,28 +1,28 @@
-"""Command Line Interface (CLI) transport for UTCP client.
+"""Command Line Interface (CLI) communication protocol for the UTCP client.
 
-This module provides the CLI transport implementation that enables UTCP clients
-to interact with command-line tools and processes. It handles tool discovery
-through startup commands, tool execution with proper argument formatting,
-and output processing with JSON parsing capabilities.
+This module provides an implementation of the `CommunicationProtocol` interface
+that enables the UTCP client to interact with command-line tools. It supports
+discovering tools by executing a command and parsing its output for a UTCP
+manual, as well as calling those tools with arguments.
 
 Key Features:
-    - Asynchronous command execution with timeout handling
-    - Tool discovery via startup commands that output UTCP manuals
-    - Flexible argument formatting for command-line flags
-    - Environment variable support for authentication and configuration
-    - JSON output parsing with fallback to raw text
-    - Cross-platform command parsing (Windows/Unix)
-    - Working directory control for command execution
+    - Asynchronous execution of shell commands.
+    - Tool discovery by running a command that outputs a UTCP manual.
+    - Flexible argument formatting for different CLI conventions.
+    - Support for environment variables and custom working directories.
+    - Automatic parsing of JSON output with a fallback to raw text.
+    - Cross-platform command parsing for Windows and Unix-like systems.
 
-Security:
-    - Command execution is isolated through subprocess
-    - Environment variables can be controlled per provider
-    - Working directory can be restricted
+Security Considerations:
+    Executing arbitrary command-line tools can be dangerous. This protocol
+    should only be used with trusted tools.
 """
 import asyncio
 import json
 import os
+import re
 import shlex
+import sys
 from typing import Dict, Any, List, Optional, Callable, AsyncGenerator
 
 from utcp.interfaces.communication_protocol import CommunicationProtocol
@@ -30,40 +30,29 @@ from utcp.data.call_template import CallTemplate, CallTemplateSerializer
 from utcp.data.tool import Tool
 from utcp.data.utcp_manual import UtcpManual, UtcpManualSerializer
 from utcp.data.register_manual_response import RegisterManualResult
-from utcp_cli.cli_call_template import CliCallTemplate, CliCallTemplateSerializer
+from utcp_cli.cli_call_template import CliCallTemplate, CliCallTemplateSerializer, CommandStep
 import logging
 
+logging.basicConfig(
+    level=logging.INFO,
+    format="%(asctime)s [%(levelname)s] %(filename)s:%(lineno)d - %(message)s"
+)
+
 logger = logging.getLogger(__name__)
 
-
 class CliCommunicationProtocol(CommunicationProtocol):
-    """Transport implementation for CLI-based tool providers.
+    """REQUIRED
+    Communication protocol for interacting with CLI-based tool providers.
 
-    Handles communication with command-line tools by executing processes
-    and managing their input/output. Supports both tool discovery and
-    execution phases with comprehensive error handling and timeout management.
-
-    Features:
-        - Asynchronous subprocess execution with proper cleanup
-        - Tool discovery through startup commands returning UTCP manuals
-        - Flexible argument formatting for various CLI conventions
-        - Environment variable injection for authentication
-        - JSON output parsing with graceful fallback to text
-        - Cross-platform command parsing and execution
-        - Configurable working directories and timeouts
-        - Process lifecycle management with proper termination
-
-    Architecture:
-        CLI tools are discovered by executing the provider's command_name
-        and parsing the output for UTCP manual JSON. Tool calls execute
-        the same command with formatted arguments and return processed output.
-
-    Attributes:
-        _log: Logger function for debugging and error reporting.
+    This class implements the `CommunicationProtocol` interface to handle
+    communication with command-line tools. It discovers tools by executing a
+    command specified in a `CliCallTemplate` and parsing the output for a UTCP
+    manual. It also executes tool calls by running the corresponding command
+    with the provided arguments.
     """
     
     def __init__(self):
-        """Initialize the CLI transport."""
+        """Initializes the `CliCommunicationProtocol`."""
     
     def _log_info(self, message: str):
         """Log informational messages."""
@@ -154,30 +143,45 @@ class CliCommunicationProtocol(CommunicationProtocol):
             raise
     
     async def register_manual(self, caller, manual_call_template: CallTemplate) -> RegisterManualResult:
-        """Register a CLI manual and discover its tools.
-        
-        Executes the call template's command_name and looks for a UTCP manual JSON in the output.
+        """REQUIRED
+        Registers a CLI-based manual and discovers its tools.
+
+        This method executes the command specified in the `CliCallTemplate`'s
+        `command_name` field. It then attempts to parse the command's output
+        (stdout) as a UTCP manual in JSON format.
+
+        Args:
+            caller: The UTCP client instance that is calling this method.
+            manual_call_template: The `CliCallTemplate` containing the details for
+                tool discovery, such as the command to run.
+
+        Returns:
+            A `RegisterManualResult` object indicating whether the registration
+            was successful and containing the discovered tools.
+
+        Raises:
+            ValueError: If the `manual_call_template` is not an instance of
+                `CliCallTemplate` or if `command_name` is not set.
         """
         if not isinstance(manual_call_template, CliCallTemplate):
             raise ValueError("CliCommunicationProtocol can only be used with CliCallTemplate")
 
-        if not manual_call_template.command_name:
-            raise ValueError(f"CliCallTemplate '{manual_call_template.name}' must have command_name set")
+        if not manual_call_template.commands:
+            raise ValueError(f"CliCallTemplate '{manual_call_template.name}' must have at least one command")
 
         self._log_info(
-            f"Registering CLI manual '{manual_call_template.name}' with command '{manual_call_template.command_name}'"
+            f"Registering CLI manual '{manual_call_template.name}' with {len(manual_call_template.commands)} command(s)"
         )
 
         try:
+            # Execute commands using the same approach as call_tool but with no arguments
             env = self._prepare_environment(manual_call_template)
-            # Parse command string into proper arguments
-            # Use posix=False on Windows, posix=True on Unix-like systems
-            command = shlex.split(manual_call_template.command_name, posix=(os.name != 'nt'))
+            shell_script = self._build_combined_shell_script(manual_call_template.commands, {})
+            
+            self._log_info(f"Executing shell script for tool discovery from provider '{manual_call_template.name}'")
 
-            self._log_info(f"Executing command for tool discovery: {' '.join(command)}")
-
-            stdout, stderr, return_code = await self._execute_command(
-                command,
+            stdout, stderr, return_code = await self._execute_shell_script(
+                shell_script,
                 env,
                 timeout=30.0,
                 working_dir=manual_call_template.working_dir,
@@ -188,14 +192,14 @@ class CliCommunicationProtocol(CommunicationProtocol):
 
             if not output.strip():
                 self._log_info(
-                    f"No output from command '{manual_call_template.command_name}'"
+                    f"No output from commands for CLI provider '{manual_call_template.name}'"
                 )
                 return RegisterManualResult(
                     success=False,
                     manual_call_template=manual_call_template,
                     manual=UtcpManual(manual_version="0.0.0", tools=[]),
                     errors=[
-                        f"No output from discovery command for CLI provider '{manual_call_template.name}'"
+                        f"No output from discovery commands for CLI provider '{manual_call_template.name}'"
                     ],
                 )
 
@@ -237,34 +241,203 @@ class CliCommunicationProtocol(CommunicationProtocol):
             )
     
     async def deregister_manual(self, caller, manual_call_template: CallTemplate) -> None:
-        """Deregister a CLI manual (no-op)."""
+        """REQUIRED
+        Deregisters a CLI manual.
+
+        For the CLI protocol, this is a no-op as there are no persistent
+        connections to terminate.
+
+        Args:
+            caller: The UTCP client instance that is calling this method.
+            manual_call_template: The call template of the manual to deregister.
+        """
         if isinstance(manual_call_template, CliCallTemplate):
             self._log_info(
                 f"Deregistering CLI manual '{manual_call_template.name}' (no-op)"
             )
     
-    def _format_arguments(self, tool_args: Dict[str, Any]) -> List[str]:
-        """Format arguments for command-line execution.
-        
-        Converts a dictionary of arguments into command-line flags and values.
+    def _substitute_utcp_args(self, command: str, tool_args: Dict[str, Any]) -> str:
+        """Substitute UTCP_ARG placeholders in command string with tool arguments.
         
         Args:
+            command: Command string containing UTCP_ARG_argname_UTCP_END placeholders
             tool_args: Dictionary of argument names and values
             
         Returns:
-            List of command-line arguments
+            Command string with placeholders replaced by actual values
         """
-        args = []
-        for key, value in tool_args.items():
-            if isinstance(value, bool):
-                if value:
-                    args.append(f"--{key}")
-            elif isinstance(value, (list, tuple)):
-                for item in value:
-                    args.extend([f"--{key}", str(item)])
+        # Pattern to match UTCP_ARG_argname_UTCP_END
+        pattern = r'UTCP_ARG_(.+?)_UTCP_END'
+        
+        def replace_placeholder(match):
+            arg_name = match.group(1)
+            if arg_name in tool_args:
+                return str(tool_args[arg_name])
             else:
-                args.extend([f"--{key}", str(value)])
-        return args
+                self._log_error(f"Missing argument '{arg_name}' for placeholder in command: {command}")
+                return f"MISSING_ARG_{arg_name}"
+        
+        return re.sub(pattern, replace_placeholder, command)
+    
+    def _build_combined_shell_script(self, commands: List[CommandStep], tool_args: Dict[str, Any]) -> str:
+        """Build a combined shell script from multiple commands.
+        
+        Args:
+            commands: List of CommandStep objects to combine
+            tool_args: Tool arguments for placeholder substitution
+            
+        Returns:
+            Shell script string that executes all commands in sequence
+        """
+        script_lines = []
+        
+        # Add error handling and setup
+        if os.name == 'nt':
+            # PowerShell script
+            script_lines.append('$ErrorActionPreference = "Stop"')  # Exit on error
+            script_lines.append('# Variables to store command outputs')
+        else:
+            # Unix shell script
+            script_lines.append('#!/bin/bash')
+            # Don't use set -e to allow error output capture and processing
+            script_lines.append('# Variables to store command outputs')
+        
+        # Execute each command and store output in variables
+        for i, command_step in enumerate(commands):
+            # Substitute UTCP_ARG placeholders
+            substituted_command = self._substitute_utcp_args(command_step.command, tool_args)
+            
+            var_name = f"CMD_{i}_OUTPUT"
+            
+            if os.name == 'nt':
+                # PowerShell - capture command output in variable
+                script_lines.append(f'${var_name} = {substituted_command} 2>&1 | Out-String')
+            else:
+                # Unix shell - capture command output in variable
+                script_lines.append(f'{var_name}=$({substituted_command} 2>&1)')
+        
+        # Echo only the outputs we want based on append_to_final_output
+        for i, command_step in enumerate(commands):
+            is_last_command = i == len(commands) - 1
+            should_append = command_step.append_to_final_output
+            
+            if should_append is None:
+                # Default: only append the last command's output
+                should_append = is_last_command
+            
+            if should_append:
+                var_name = f"CMD_{i}_OUTPUT"
+                if os.name == 'nt':
+                    # PowerShell
+                    script_lines.append(f'Write-Output ${var_name}')
+                else:
+                    # Unix shell
+                    script_lines.append(f'echo "${{{var_name}}}"')
+        
+        return '\n'.join(script_lines)
+    
+    async def _execute_shell_script(self, script: str, env: Dict[str, str], timeout: float = 60.0, working_dir: Optional[str] = None) -> tuple[str, str, int]:
+        """Execute a shell script in a single subprocess.
+        
+        Args:
+            script: Shell script content to execute
+            env: Environment variables
+            timeout: Timeout in seconds
+            working_dir: Working directory for script execution
+            
+        Returns:
+            Tuple of (stdout, stderr, return_code)
+        """
+        process = None
+        try:
+            # Choose shell based on OS
+            if os.name == 'nt':
+                # Windows: use PowerShell
+                shell_cmd = ['powershell.exe', '-Command']
+            else:
+                # Unix: use bash
+                shell_cmd = ['/bin/bash', '-c']
+            
+            # Add the script as the last argument
+            full_command = shell_cmd + [script]
+            
+            process = await asyncio.create_subprocess_exec(
+                *full_command,
+                stdout=asyncio.subprocess.PIPE,
+                stderr=asyncio.subprocess.PIPE,
+                env=env,
+                cwd=working_dir
+            )
+            
+            stdout_bytes, stderr_bytes = await asyncio.wait_for(
+                process.communicate(),
+                timeout=timeout
+            )
+            
+            stdout = stdout_bytes.decode('utf-8', errors='replace')
+            stderr = stderr_bytes.decode('utf-8', errors='replace')
+            
+            return stdout, stderr, process.returncode or 0
+            
+        except asyncio.TimeoutError:
+            if process:
+                try:
+                    process.kill()
+                    await process.wait()
+                except ProcessLookupError:
+                    pass
+            self._log_error(f"Shell script timed out after {timeout} seconds")
+            raise
+        except Exception as e:
+            if process:
+                try:
+                    process.kill()
+                    await process.wait()
+                except ProcessLookupError:
+                    pass
+            self._log_error(f"Error executing shell script: {e}")
+            raise
+    
+    def _parse_combined_output(self, stdout: str, stderr: str, return_code: int, commands: List[CommandStep], tool_name: str) -> Any:
+        """Parse output from combined shell script execution.
+        
+        Args:
+            stdout: Standard output from shell script
+            stderr: Standard error from shell script
+            return_code: Exit code of shell script
+            commands: Original commands list for output control (unused with variable approach)
+            tool_name: Name of the tool for logging
+            
+        Returns:
+            Final output from script (already filtered by append_to_final_output)
+        """
+        # Platform-specific output handling
+        if os.name == 'nt':
+            # Windows (PowerShell): Use stdout on success, stderr on failure
+            output = stdout if return_code == 0 else stderr
+        else:
+            # Unix (Bash): Our script captures everything and echoes to stdout
+            # So we always use stdout first, fallback to stderr if stdout is empty
+            output = stdout if stdout.strip() else stderr
+        
+        if not output.strip():
+            self._log_info(f"CLI tool '{tool_name}' produced no output")
+            return ""
+        
+        # With the variable approach, output is already filtered - just return it
+        output = output.strip()
+        
+        # Try to parse as JSON if it looks like JSON
+        if output.startswith(('{', '[')):
+            try:
+                result = json.loads(output)
+                self._log_info(f"Returning JSON output from CLI tool '{tool_name}'")
+                return result
+            except json.JSONDecodeError:
+                pass
+        
+        self._log_info(f"Returning text output from CLI tool '{tool_name}'")
+        return output
     
     def _extract_utcp_manual_from_output(self, output: str, provider_name: str) -> Optional[UtcpManual]:
         """Extract a UTCP manual from command output.
@@ -399,84 +572,67 @@ class CliCommunicationProtocol(CommunicationProtocol):
         return tools
     
     async def call_tool(self, caller, tool_name: str, tool_args: Dict[str, Any], tool_call_template: CallTemplate) -> Any:
-        """Call a CLI tool.
-        
-        Executes the command specified by provider.command_name with the provided arguments.
-        
+        """REQUIRED
+        Calls a CLI tool by executing its command.
+
+        This method constructs and executes the command specified in the
+        `CliCallTemplate`. It formats the provided `tool_args` as command-line
+        arguments and runs the command in a subprocess.
+
         Args:
-            caller: The UTCP client that is calling this method.
-            tool_name: Name of the tool to call
-            tool_args: Arguments for the tool call
-            tool_call_template: The CliCallTemplate for the tool
-            
+            caller: The UTCP client instance that is calling this method.
+            tool_name: The name of the tool to call.
+            tool_args: A dictionary of arguments for the tool call.
+            tool_call_template: The `CliCallTemplate` for the tool.
+
         Returns:
-            The output from the command execution based on exit code:
-            - If exit code is 0: stdout (parsed as JSON if possible, otherwise raw string)
-            - If exit code is not 0: stderr
-            
+            The result of the command execution. If the command exits with a code
+            of 0, it returns the content of stdout. If the exit code is non-zero,
+            it returns the content of stderr. The output is parsed as JSON if
+            possible; otherwise, it is returned as a raw string.
+
         Raises:
-            ValueError: If provider is not a CliProvider or command_name is not set
+            ValueError: If `tool_call_template` is not an instance of
+                `CliCallTemplate` or if `command_name` is not set.
         """
         if not isinstance(tool_call_template, CliCallTemplate):
             raise ValueError("CliCommunicationProtocol can only be used with CliCallTemplate")
         
-        if not tool_call_template.command_name:
-            raise ValueError(f"CliCallTemplate '{tool_call_template.name}' must have command_name set")
+        if not tool_call_template.commands:
+            raise ValueError(f"CliCallTemplate '{tool_call_template.name}' must have at least one command")
         
-        # Build the command
-        # Parse command string into proper arguments
-        # Use posix=False on Windows, posix=True on Unix-like systems
-        command = shlex.split(tool_call_template.command_name, posix=(os.name != 'nt'))
-        
-        # Add formatted arguments
-        if tool_args:
-            command.extend(self._format_arguments(tool_args))
-        
-        self._log_info(f"Executing CLI tool '{tool_name}': {' '.join(command)}")
+        self._log_info(f"Executing CLI tool '{tool_name}' with {len(tool_call_template.commands)} command(s) in single subprocess")
         
         try:
             env = self._prepare_environment(tool_call_template)
             
-            stdout, stderr, return_code = await self._execute_command(
-                command,
+            # Build combined shell script with output capture
+            shell_script = self._build_combined_shell_script(tool_call_template.commands, tool_args)
+            
+            self._log_info("Executing combined shell script")
+            
+            # Execute the combined script in a single subprocess
+            stdout, stderr, return_code = await self._execute_shell_script(
+                shell_script,
                 env,
-                timeout=60.0,  # Longer timeout for tool execution
+                timeout=120.0,  # Longer timeout for multi-command execution
                 working_dir=tool_call_template.working_dir
             )
             
-            # Get output based on exit code
-            if return_code == 0:
-                output = stdout
-                self._log_info(f"CLI tool '{tool_name}' executed successfully (exit code 0)")
-            else:
-                output = stderr
-                self._log_info(f"CLI tool '{tool_name}' exited with code {return_code}, returning stderr")
+            # Parse the output to extract individual command outputs
+            final_output = self._parse_combined_output(stdout, stderr, return_code, tool_call_template.commands, tool_name)
             
-            # Try to parse output as JSON, fall back to raw string
-            if output.strip():
-                try:
-                    result = json.loads(output)
-                    self._log_info(f"Returning JSON output from CLI tool '{tool_name}'")
-                    return result
-                except json.JSONDecodeError:
-                    # Return raw string output
-                    self._log_info(f"Returning text output from CLI tool '{tool_name}'")
-                    return output.strip()
-            else:
-                self._log_info(f"CLI tool '{tool_name}' produced no output")
-                return ""
+            return final_output
             
         except Exception as e:
             self._log_error(f"Error executing CLI tool '{tool_name}': {e}")
             raise
 
     async def call_tool_streaming(self, caller, tool_name: str, tool_args: Dict[str, Any], tool_call_template: CallTemplate) -> AsyncGenerator[Any, None]:
-        """Streaming calls are not supported for CLI protocol."""
-        raise NotImplementedError("Streaming is not supported by the CLI communication protocol.")
-    
-    async def close(self) -> None:
-        """Close the transport.
-        
-        This is a no-op for CLI transports since they don't maintain connections.
+        """REQUIRED
+        Streaming calls are not supported for the CLI protocol.
+
+        Raises:
+            NotImplementedError: Always, as this functionality is not supported.
         """
-        self._log_info("Closing CLI transport (no-op)")
+        raise NotImplementedError("Streaming is not supported by the CLI communication protocol.")
